/////////////////////////////////////////////////////////////////////////
// $Id: packageDisplay.h,v 1.12 2003/09/17 16:34:21 toivo Exp $	
// File  : packageDisplay.h
// Author: Toivo Pedaste
//
// This widget is used to display information and the file list of a
// package.
//
// Package information will be displayed using (another) sub-widget
// that is inherited from a QTableView.
//
// The file list will be displayed using a tree list.
//
// The widget is mainly a QTabDialog with two tabs: Info and FileList.
// The Info tab is the default one.
//
/////////////////////////////////////////////////////////////////////////

#ifndef PACKAGEDISPLAY_H
#define PACKAGEDISPLAY_H
#include "../config.h"

// Qt Headers
#include <qframe.h>
#include <qtabbar.h>
#include <qtabwidget.h>
#include <qvbox.h>
 

class packagePropertiesWidget;
class packageInfo;
class KListView;
class QListViewItem;
class QTextEdit;

class packageDisplayWidget : public QTabWidget
{
  Q_OBJECT

  ///////////// METHODS ------------------------------------------------------
public:
  packageDisplayWidget(QWidget *parent=0);
  // Constructor

  ~packageDisplayWidget();
  // Destructor

  void noPackage();
  // clear package display in right panel

  void changePackage(packageInfo *p);
  // Set currently selected package

private:
  void setupWidgets();
  // This sets up the sub-widgets

  void updateFileList();
  // This updates the file list to match that found with the currently
  // selected package
  
  void updateChangeLog();
  // This updates the change log to match that found with the currently
  // selected package
  
  void tabSet(QWidget *);
  // Set display for corresponding tab

  ///////////// SLOTS --------------------------------------------------------
public slots:

  void tabSelected(QWidget *);

  void openBinding(QListViewItem *);

  ///////////// SIGNALS ------------------------------------------------------

  ///////////// DATA ---------------------------------------------------------
public:
  packageInfo *package;
  // the currently selected package

private:

  QTabWidget *tabbar;
  // The tab bar

  QVBox *proptab, *fltab, *cltab;

  QWidget *curTab;
  // current active tab

  KListView *fileList;
  // This holds the file list (and is used as a page on the tab dialog)

  QTextEdit *changeLog;
  // Holds changelog

  QPixmap tick, cross, question, *blank;
  // The pixmaps for the filelist

  packagePropertiesWidget *packageProperties;
  // This displays the package properties (and is used as a page on the
  // tab dialog)

  bool initList;
  // True is file list has been initialised
};
#endif
